-- MinorCivEvolved
-- Author: Charsi
-- DateCreated: 03/13/2016 8:26:39 PM
--------------------------------------------------------------

function MinorCivEvolvedStartTurn(iPlayer)
	local pMinor = Players[iPlayer]
	local pActivePlayer = Players[Game.GetActivePlayer()]
	local iQuestData1, iQuestData2

	if (pMinor == nil) then
		print("MinorCivEvolvedStartTurn: aborting for [" .. iPlayer .. "] because player object is null.")
		return
	end

	if Teams[pMinor:GetTeam()] == nil then
		print("MinorCivEvolvedStartTurn: aborting for [" .. iPlayer .. "] because team object is null.")
		return
	end

	if not Teams[pMinor:GetTeam()]:IsAlive() then
		print("MinorCivEvolvedStartTurn: aborting for [" .. iPlayer .. "] because team is not alive.")
		return
	end

	-- print("MinorCivEvolvedStartTurn: Processing for [" .. iPlayer .. "].");

	-- only iterate for minor civs
	if pMinor:IsMinorCiv() then
		-- Based on type, grant free buildings
		iTrait = pMinor:GetMinorCivTrait()
		iBuilding = -1;

		if (iTrait == GameInfoTypes.MINOR_TRAIT_CULTURED) then
			-- Free Monument (+2c)
			iBuilding = GameInfoTypes.BUILDING_MONUMENT
			-- print("Minor civ [" .. iPlayer .. "], [" .. pMinor:GetName() .."] is type [" .. iTrait .. "] --> Monument (+2c)")
		end

		if (iTrait == GameInfoTypes.MINOR_TRAIT_MILITARISTIC) then
			-- Free Barracks (+15xp)
			iBuilding = GameInfoTypes.BUILDING_BARRACKS
			-- print("Minor civ [" .. iPlayer .. "], [" .. pMinor:GetName() .."] is type [" .. iTrait .. "] --> Barracks (+15xp)")
		end

		if (iTrait == GameInfoTypes.MINOR_TRAIT_MARITIME) then
			-- Free Lighthouse (+1f)
			iBuilding = GameInfoTypes.BUILDING_LIGHTHOUSE
			-- print("Minor civ [" .. iPlayer .. "], [" .. pMinor:GetName() .."] is type [" .. iTrait .. "] --> Lighthouse (+1f)")
		end

		if (iTrait == GameInfoTypes.MINOR_TRAIT_MERCANTILE) then
			-- Free Market (+1g +25%g)
			iBuilding = GameInfoTypes.BUILDING_MARKET
			-- print("Minor civ [" .. iPlayer .. "], [" .. pMinor:GetName() .."] is type [" .. iTrait .. "] --> Market (1g +25%g)")
		end

		if (iTrait == GameInfoTypes.MINOR_TRAIT_RELIGIOUS) then
			-- Free Pyramid (+2f/2s)
			iBuilding = GameInfoTypes.BUILDING_MAYA_PYRAMID
			-- print("Minor civ [" .. iPlayer .. "], [" .. pMinor:GetName() .."] is type [" .. iTrait .. "] --> Pyramid (2f/2s)")
		end

		for pMinorCity in pMinor:Cities() do
			if (iBuilding ~= -1) then
				if not pMinorCity:IsHasBuilding(iBuilding) then
					pMinorCity:SetNumRealBuilding(iBuilding, 1)
				end
			end
			-- new: give them the faux "nation state presence" building
			if not pMinorCity:IsHasBuilding(GameInfoTypes.BUILDING_NATION_STATE_PRESENCE) then
				pMinorCity:SetNumRealBuilding(GameInfoTypes.BUILDING_NATION_STATE_PRESENCE, 1)
			end
		end

		-- print("MinorCivEvolvedStartTurn: Iterating minors, checking potential for peace.");
		-- iterate over all minors.. make peace (small percentage chance)
		for j = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS - 1, 1 do
			if Players[j]:IsAlive() and (iPlayer ~= j) then
				local teamMinorInstigator = Teams[pMinor:GetTeam()]
				local iMinorVictimTeam = Players[j]:GetTeam()

				if teamMinorInstigator:CanChangeWarPeace(iMinorVictimTeam) then
					if teamMinorInstigator:IsAtWar(iMinorVictimTeam) then
						-- print("MinorCivEvolvedStartTurn: State of war detected between Instigator [" .. iPlayer .. "|" .. pMinor:GetTeam() .. "|" .. pMinor:GetName() .. "] and Victim [" .. j .. "|" .. Players[j]:GetTeam() .. "|" .. Players[j]:GetName() .. "].");
						if (teamMinorInstigator:GetNumTurnsLockedIntoWar(iMinorVictimTeam) <= 0) then
							-- print("MinorCivEvolvedStartTurn: At war, and no longer locked in.  Checking if peace can be made between Instigator and Victim.");
							local iChance = math.random(100)

							-- 1% chance (i.e. very small)
							if (iChance < 2) then
								print("MinorCivEvolvedStartTurn: Dice roll succeeded [" .. iChance .. "].  Setting peace state between Instigator and Victim.");
								teamMinorInstigator:MakePeace(iMinorVictimTeam)

								-- local pActivePlayer = Players[Game.GetActivePlayer()]
								-- local sTitle = pMinor:GetName() .. "" .. Players[j]:GetName() .. "Ƙa!"
								-- local sText = "킢̖A".. pMinor:GetName() .. "" .. Players[j]:GetName() .. "Ƙa!"

								-- pActivePlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText, sTitle, pMinor:GetCapitalCity():GetX(), pMinor:GetCapitalCity():GetY())
							else
								-- print("MinorCivEvolvedStartTurn: Dice roll failed [" .. iChance .. "].  Skipped.");
							end
						else
							-- print("MinorCivEvolvedStartTurn: At war, and locked in (GetNumTurnsLockedIntoWar > 0).  Skipped.");
						end
					end
				else
					-- print("MinorCivEvolvedStartTurn: Instigator [" .. iPlayer .. "] cannot change war state with Victim [" .. j .."].  Skipped.");
				end
			end
		end

		-- print("MinorCivEvolvedStartTurn: Iterating majors, checking potential for war.");
		-- determine if we should declare war by looking for offered "bullying" quests
		i = Game.GetActivePlayer()
		-- for i = 0, GameDefines.MAX_MAJOR_CIVS - 1, 1 do
			if pMinor:IsMinorCivActiveQuestForPlayer(i, MinorCivQuestTypes.MINOR_CIV_QUEST_BULLY_CITY_STATE) then
				iQuestData1 = pMinor:GetQuestData1(i, MinorCivQuestTypes.MINOR_CIV_QUEST_BULLY_CITY_STATE)
				iQuestData2 = pMinor:GetQuestData2(i, MinorCivQuestTypes.MINOR_CIV_QUEST_BULLY_CITY_STATE)

				-- print("MinorCivEvolvedStartTurn: Instigator [" .. iPlayer .. "|" .. pMinor:GetTeam() .. "|" .. pMinor:GetName() .. "] wants Victim [" .. iQuestData1 ..  "|" .. Players[iQuestData1]:GetTeam() .. "|" .. Players[iQuestData1]:GetName() .. "] to be bullied by Major [" .. i .. " - " .. Players[i]:GetName() .. "] - note param2 [" .. iQuestData2 .."].")

				local teamMinorInstigator = Teams[pMinor:GetTeam()]
				local iMinorVictimTeam = Players[iQuestData1]:GetTeam()

				if teamMinorInstigator:CanChangeWarPeace(iMinorVictimTeam) then
					-- print("MinorCivEvolvedStartTurn: Instigator can change war state with Victim.  Performing checks...");
					if teamMinorInstigator:IsAtWar(iMinorVictimTeam) then
						-- print("MinorCivEvolvedStartTurn: Yup, they're at war.");
					else
						if teamMinorInstigator:CanDeclareWar(iMinorVictimTeam) then
							local iTurnsRemaining = pMinor:GetQuestTurnsRemaining(i, MinorCivQuestTypes.MINOR_CIV_QUEST_BULLY_CITY_STATE, Game.GetGameTurn() - 1);

							-- 100% chance on final turn turn
							if (iTurnsRemaining >= 1) then
								print("MinorCivEvolvedStartTurn: [" .. iTurnsRemaining .. "] turn remaining.  City State has become impatient - setting war state between Instigator and Victim.");
								teamMinorInstigator:DeclareWar(iMinorVictimTeam)

								-- local pActivePlayer = Players[Game.GetActivePlayer()]
								-- local sTitle = pMinor:GetName() .. "" .. Players[iQuestData1]:GetName() .. "ɐz!"
								-- local sText = Players[iQuestData1]:GetName() .. "ɒł悤߂Ă" .. pMinor:GetName() .. "z!"

								-- pActivePlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText, sTitle, pMinor:GetCapitalCity():GetX(), pMinor:GetCapitalCity():GetY())
							else
								-- print("MinorCivEvolvedStartTurn: [" .. iTurnsRemaining .. "] turns remaining.  City State continues to waits patiently.");
							end
						else
							-- print("MinorCivEvolvedStartTurn: Instigator cannot declare war on Victim.  Skipped.");
						end
					end
				else
					-- print("MinorCivEvolvedStartTurn: Instigator cannot change war state with Victim.  Skipped.");
				end
			end
		-- end
	end
end

function MinorCivEvolvedLimitSettlers(iPlayer, iUnitType)
	local pMinor = Players[iPlayer]
	local retval = true

	if pMinor:IsMinorCiv() then
		if (iUnitType == GameInfoTypes.UNIT_SETTLER) then
			local iNumCities = pMinor:GetNumCities()
			local iNumSettlers = pMinor:GetUnitClassCountPlusMaking(GameInfoTypes.UNITCLASS_SETTLER)
			local iPotentialCities = iNumCities + iNumSettlers
			local iCityCap = pMinor:GetCurrentEra() + 2

			-- print("MinorCivEvolvedLimitSettlers: [" .. pMinor:GetName() .. "] Potential Cities [" .. iPotentialCities .. "] Era+2 [" .. iCityCap .."]")

			-- limit of 2 (cities + settlers) in ancient era; add 1 per era
			if (iPotentialCities >= iCityCap) then
				-- print(pMinor:GetName() .. " is at the City + Settler cap of [" .. iCityCap .. "]; Settler choice is hidden.")
				retval = false
			end
		end
	end

	return retval
end

function MinorCivEvolvedLimitSettlersByCity(iPlayer, iCity, iUnitType)
	return MinorCivEvolvedLimitSettlers(iPlayer, iUnitType)
end

-- Don't like settler limits?  Comment these two lines out by putting two dashes in front of them.

-- GameEvents.PlayerCanTrain.Add(MinorCivEvolvedLimitSettlers);
-- GameEvents.CityCanTrain.Add(MinorCivEvolvedLimitSettlersByCity);

-- Don't like city states declaring war?  Comment this line out by putting two dashes in front of it.

GameEvents.PlayerDoTurn.Add(MinorCivEvolvedStartTurn);